#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 06-Mar-2004
################################################################################
#
# Contributros - Secondary access portal designed to be used by contributors to
#                access all of their specific features.
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use Number::Format qw(:subs);					# Number formatting library
use strict;							# Strict variable enforcement

use eRealms::Database;						# Database handler
use eRealms::Filter;						# Filter handling
use eRealms::Home;						# Homepage handling
use eRealms::Login;						# Login functionality
use eRealms::Mail;						# Mail handler
use eRealms::Option;						# Option handler
use eRealms::Param;						# Parameter control
use eRealms::Table;						# Table handler
use eRealms::Template;						# Template handler


################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter hash
my $tmpl;							# Template handle

my $portal;							# Portal template
my $key;							# Selected key	

my %contrib;							# Contributors hash
my %gal;							# Gallery hash
my %srch;							# Built up macros list

################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new eRealms::Database();

	# Connect and fetch
	$database->Connect();

	# Parameter handling
	$param    = new eRealms::Param($database, $cgi);
	$param->GetParam();

	# Link with hash
	$database->GetHashContrib(\%contrib);
	$database->GetHashGallery(\%gal);


	# Authenthication instance
	$login = new eRealms::Login($database, $cgi, $param, \%gal);

	# Templace instance
	$tmpl = new eRealms::Template(\%contrib);



	# Document header
	print $cgi->header();
	$database->DocumentGetContribHeader();

	# Authenthication
	if ($login->GetVerificationNormal())
	{
		# Collect information
		CollectGeneral($database, $param, \%contrib, \%srch);
		CollectHome($database, $param, $tmpl, \%contrib, \%srch);
		CollectGallery($database, $param, $tmpl, \%contrib, \%srch);	

		# Pull portal
		$portal = $database->DocumentGetContribPortal();


		# Search and replace
		foreach $key (sort(keys(%srch)))
		{
			# Regular expression
			$portal =~ s/$key/$srch{$key}/gs;
		}

		# Display completed portal
		print "\n$portal\n";
	}

	# Document footer
	$database->DocumentGetContribFooter();


################################################################################
# Sub-Routines
################################################################################

#####################
# Gallery Collection
#
# This subroutine will generates lists of all relevant galleries for a specific
# type.   Going through all three, this will generate the final macros needed
# for the template generation.

sub CollectGallery
{
	#####################
	# Data members

	my $database = shift;					# Database handler
	my $param    = shift;					# Parameter handle
	my $tmpl     = shift;					# Template handle
	my $contrib  = shift;					# Contributor hash
	my $srch     = shift;					# Search and replace hash

	my $res;						# Results string
	my $statement;						# Database statement
	
	my $iview;						# Inline parameters
	my $imanage;						# Inline parameters

	my $genres;						# Formatted genres
	my $priv;						# Privacy features
	my $type;						# Singular gallery type
	my $work;						# Amount of works

	my @genres;						# Genres for a gallery
	my @types    = qw(COMIC IMAGE JOURNAL TEXT);		# Types of galleries


	#####################
	# Program area

	# Viewing inline parameters
	$iview = $param->EmbedInline(
 	  USER         => $param->{'USER'}, 
 	  CRYPT        => $param->{'CRYPT'}
	);
					
	# Global define
	$srch->{'MGALVIEW'}   = $contrib->{'LnkView'} . $iview;
	$srch->{'MGALMANAGE'} = $contrib->{'LnkManage'} . $iview;

	
	# Generate all types of lists
	foreach $type (@types)
	{
		# Initialize counter
		$srch->{"M${type}COUNT"} = 0;

		# Create entries
		{
			# Search for relevant material
			$database->Pull(\$statement, "SELECT
				 GalleryID       AS \"GalleryID\",
				 GalleryName     AS \"GalleryName\",
				 GalleryPenName  AS \"GalleryPenName\",
				 GalleryRating   AS \"GalleryRating\",
				 GalleryPrivacy  AS \"GalleryPrivacy\",
				 getDate(GalleryCreated) AS \"GalleryCreated\",
				 getDate(GalleryUpdated) AS \"GalleryUpdated\"
				FROM Gallery
				WHERE PuppeteerLogin=?
				  AND GalleryType=?
				ORDER BY GalleryID",

			 $param->{'USER'},
			 lc($type)
			);

			# Verification
			if ($res = $statement->fetchrow_hashref())
			{
				# Initialize list
				$srch->{"LST${type}"} = '';
				
				# Loop and adjust
				do
				{
					# Increment
					$srch->{"M${type}COUNT"}++;

				
					# Genre searching	
					$database->GetListEstablished(\@genres, $res->{'GalleryID'});

					# Format genre line
					$genres = join("/", @genres);


					# Adjust privacy
					$priv   = ($res->{'GalleryPrivacy'} eq 'yes') ? $contrib->{'TagPrivTrue'} : $contrib->{'TagPrivFalse'};

					# Count works
					($work) = $database->DataGet("SELECT COUNT(*)
						FROM Work
						WHERE GalleryID=?
						  AND WorkID > 1",

					 $res->{'GalleryID'}
					);
					    	
					
					
					# Links
					# Management inline parameters
					$imanage = $param->EmbedInline(
	  				  USER         => $param->{'USER'}, 
	  				  CRYPT        => $param->{'CRYPT'},
					  GALLERY      => $res->{'GalleryID'} . ' - ' . $res->{'GalleryName'},
					  GID          => $res->{'GalleryID'},
					  EDIT_GALLERY => 'False'
					);
					

					# Append to list
					$srch->{"LST${type}"} = $srch->{"LST${type}"}
				       	. $tmpl->Pass('TmplGalleryList',
						MCREATE => $res->{'GalleryCreated'},
						MID     => $res->{'GalleryID'},
						MGNAME  => $res->{'GalleryName'},
						MPNAME  => $res->{'GalleryPenName'},
						MRATE   => $res->{'GalleryRating'},
						MUPDATE => $res->{'GalleryUpdated'},
						MGENRES => $genres,
						MPRIV   => $priv,
						MWRKCNT => $work,
						LVIEW   => $contrib->{'LnkView'} . '/view/' . $res->{'GalleryID'} . $iview,
						LMANAGE => $contrib->{'LnkManage'} . $imanage);

				} while ($res = $statement->fetchrow_hashref());
			}

			# Nothing to display
			else
			{
				$srch->{"LST${type}"} = $contrib->{'Tmpl' . ucfirst(lc($type)) . 'Missing'};
			}
		}
	}
}

	
#####################
# General information
#
# Will gather general information specific to the user and seperate from the
# homepage or gallery components that are covered elsewhere.

sub CollectGeneral 
{
	#####################
	# Data members

	my $database = shift;					# Database handler
	my $param    = shift;					# Parameters
	my $contrib  = shift;					# Contributor hash
	my $srch     = shift;					# Search and replace hash


	#####################
	# Program area

	# Pull information
	($srch->{'MGENUSER'},
	 $srch->{'MGENNAME'},
	 $srch->{'MGENEMAIL'},
	 $srch->{'MGENVERIFY'},
	 $srch->{'MGENEXPIRE'}
 	) = $database->DataGet("SELECT
		 PuppeteerLogin     AS \"PuppeteerLogin\",
		 PuppeteerName      AS \"PuppeteerName\",
		 PuppeteerEmail     AS \"PuppeteerEmail\",
		 PuppeteerVerified  AS \"PuppeteerVerified\",
		 getDate(PuppeteerHomeSpan) AS \"PuppeteerHomeSpan\"
		FROM Puppeteer
       		WHERE PuppeteerLogin=?", $param->{'USER'});

	# Verified user formatting
	$srch->{'MGENVERIFY'} = ($srch->{'MGENVERIFY'} eq 'yes') ? $contrib->{'TagVerTrue'} : $contrib->{'TagVerFalse'};

	# General inline parameters
	$srch->{'LGENPARAM'} = $param->EmbedInline(
 	  USER         => $param->{'USER'}, 
 	  CRYPT        => $param->{'CRYPT'}
	);
}


#####################
# Homepage Collect 
#
# Homepage collection is a sub-routine that is designed to populate the hash
# with all of the necessary information in order to send off it's results to
# the template handler.

sub CollectHome
{
	#####################
	# Data members

	my $database = shift;					# Database handler
	my $param    = shift;					# Parameters
	my $tmpl     = shift;					# Template handler
	my $contrib  = shift;					# Contributor hash
	my $srch     = shift;					# Search and replace hash

	my $hp;							# Homepage handle
	my $inline;						# Inline management
	my $file;						# Singular file
	my $remain;						# Remaining space

	my @files;						# Pulled filenames
	my %home;						# Homepage hash


	#####################
	# Program area

	# Populate hash
	$database->GetHashHome(\%home);
	
	# Create instances
	$hp = new eRealms::Home($database, $home{'SetRoot'});
	
	# Inline parameters
	$inline = $param->EmbedInline(
	  USER  => $param->{'USER'}, 
	  CRYPT => $param->{'CRYPT'}
 	 );

	
	# Find and home directory
	($srch->{'MHPHOME'}) = $database->DataGetPuppeteerHome($param->{'USER'});

	# Homepage management link
	$srch->{'LHPMANAGE'} = $contrib->{'LnkHomepage'} . $inline;

	# File limits
	$srch->{'MHPLIMIT'}  = format_bytes(($home{'SetSize'} * 1048576), 2); 


	# Check for existence
	# Defined
	if (defined($srch->{'MHPHOME'}))
	{
		# Base information
		# Used space and count
		($srch->{'MHPSIZE'},
		 $srch->{'MHPCOUNT'}) = $hp->FileListing($srch->{'MHPHOME'}, \@files, $hp);

	 	# Remaining size calculation
	 	$remain = ($home{'SetSize'} * 1048576) - $srch->{'MHPSIZE'};
		$remain = ($remain < 0) ? 0 : $remain;
		
	 	# Format accordingly
		$srch->{'MHPSIZE'}  = format_bytes($srch->{'MHPSIZE'}, 2);
		$srch->{'MHPAVAIL'} = format_bytes($remain, 2);
		$srch->{'MHPCOUNT'} = format_number($srch->{'MHPCOUNT'},0,0);
	 
	 	# Generate list
		# Initial assignment
		$srch->{'LSTHPFILES'} = '';
		
		# Loop and generate
		foreach $file (sort(@files))
		{
			# Append new works
			$srch->{'LSTHPFILES'} = $srch->{'LSTHPFILES'}
		       	. $tmpl->Pass('TmplFileList',
			       	MFNAME=> $file,
				MFLINK=> $home{'SetUrl'} . $srch->{'MHPHOME'} . '/' . $file);
		}
	}

	# Undefined
	else
	{
		# Define to base values
		$srch->{'MHPCOUNT'}   = $contrib->{'TextNA'};
		$srch->{'MHPHOME'}    = $contrib->{'TextHome'};
		$srch->{'MHPSIZE'}    = $contrib->{'TextNA'};

		# List cheat
		$srch->{'LSTHPFILES'} = $contrib->{'TagNoFiles'};
	}

	# Return true
	return 1;
}
